sr=44100
kr=44100
ksmps=1
nchnls=2

instr 1
; This simulates a planet orbiting in a multiple star system.

  kampenv linseg 0, .01, p4, p3-.02, p4, .01, 0
  kmassenv linseg .1,p3/16,1,p3/8,0,p3/4,0

; Planet Position (X, Y, Z) & Velocity (VX, VY, VZ)
  kx  init 0
  ky  init .1
  kz  init 0
  kvx init .5
  kvy init .6
  kvz init -.1

; Star 1 Mass & X, Y, Z
  imass1 = p5
  is1x   init 0
  is1y   init 0
  is1z   init p7

; Star 2 Mass & X, Y, Z
  kmass2 = p6*kmassenv
  is2x   init 0
  is2y   init 0
  is2z   init -p7

; Calculate distance to Star 1
  kdx=is1x-kx
  kdy=is1y-ky
  kdz=is1z-kz
  ksqradius=kdx*kdx+kdy*kdy+kdz*kdz+1
  kradius=sqrt(ksqradius)

; Determine acceleration due to Star 1 (AX, AY, AZ)
  kax=imass1/ksqradius*kdx/kradius
  kay=imass1/ksqradius*kdy/kradius
  kaz=imass1/ksqradius*kdz/kradius

; Calculate distance to Star 2
  kdx=is2x-kx
  kdy=is2y-ky
  kdz=is2z-kz
  ksqradius=kdx*kdx+kdy*kdy+kdz*kdz+1
  kradius=sqrt(ksqradius)

; Determine acceleration due to Star 2 (AX, AY, AZ)
  kax=kax+kmass2/ksqradius*kdx/kradius
  kay=kay+kmass2/ksqradius*kdy/kradius
  kaz=kaz+kmass2/ksqradius*kdz/kradius

; Update the velocity
  kvx=kvx+kax
  kvy=kvy+kay
  kvz=kvz+kaz

; Update the position
  kx=kx+kvx
  ky=ky+kvy
  kz=kz+kvz

  aoutx = kx*kampenv
  aouty = ky*kampenv
  outs aoutx, aouty

endin

instr 2
; This simulates a planet orbiting in a multiple star system.

  kampenv linseg 0, .1, p4, p3-.2, p4, .1, 0

; Planet Position (X, Y, Z) & Velocity (VX, VY, VZ)
  kx  init 0
  ky  init 10
  kz  init 0
  kvx init 110
  kvy init 120
  kvz init -200

  ifriction = .9

; Star 1 Mass & X, Y, Z
  imass1 init p5
  is1x   init 0
  is1y   init 0
  is1z   init p7

; Star 2 Mass & X, Y, Z
  imass2 init p6
  is2x   init 0
  is2y   init 0
  is2z   init -p7

; Calculate distance to Star 1
  kdx=is1x-kx
  kdy=is1y-ky
  kdz=is1z-kz
  ksqradius=kdx*kdx+kdy*kdy+kdz*kdz+1

; Determine acceleration due to Star 1 (AX, AY, AZ)
  kax=imass1/ksqradius*kdx
  kay=imass1/ksqradius*kdy
  kaz=imass1/ksqradius*kdz

; Calculate distance to Star 2
  kdx=is2x-kx
  kdy=is2y-ky
  kdz=is2z-kz
  ksqradius=kdx*kdx+kdy*kdy+kdz*kdz+1

; Determine acceleration due to Star 2 (AX, AY, AZ)
  kax=kax+imass2/ksqradius*kdx
  kay=kay+imass2/ksqradius*kdy
  kaz=kaz+imass2/ksqradius*kdz

; Update the velocity
  kvx=ifriction*kvx+kax
  kvy=ifriction*kvy+kay
  kvz=ifriction*kvz+kaz

; Update the position
  kx=kx+kvx
  ky=ky+kvy
  kz=kz+kvz

  aoutx = kx*kampenv
  aouty = ky*kampenv
  outs aoutx, aouty

endin

instr 3 ; Hypocycloid or Spirograph curve
        ; This set of parametric equations defines the path traced by
        ; a point on a circle of radius B rotating inside a circle of
        ; radius A.

;kphi init 0

ifqc = cpspch(p5)
ia   = p6
ib   = p7

kampenv linseg 0, .1, p4, p3-.2, p4, .1, 0
kptchenv linseg .1, .2*p3, 1, .8*p3, 1
kvibenv  linseg 0, .3*p3, 0, .2*p3, 1, .5*p3, 1
kvibr oscil 20, 8, 1
kfqc = ifqc*kptchenv+kvibr*kvibenv

; CAUTION:This may overflow on sustained notes
acos1 oscil ia-ib, kfqc, 1, .25
acos2 oscil ib, (ia-ib)/ib*kfqc, 1, .25
ax = acos1 + acos2

asin1 oscil ia-ib, kfqc, 1
asin2 oscil ib, (ia-ib)/ib*kfqc, 1
ay = asin1 - asin2

;kx = (ia-ib)*cos(kphi) + ib*cos((ia-ib)/ib*kphi)
;ky = (ia-ib)*sin(kphi) - ib*sin((ia-ib)/ib*kphi)
;ax = kx
;ay = ky
outs kampenv*ax, kampenv*ay

endin