; ************************************************************************
; Advanced Resonant Low-Pass Filter Instruments
;*************************************************************************
; 1. FM Based Rezzy Synth
; 2. 4 Pole Rezzy
; 3. 6 Pole Formant Filter
;*************************************************************************

sr     = 44100
kr     = 44100
ksmps  = 1
nchnls = 2

;******************************************************************
instr 1   ; FM Based Rezzy Synth

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez   = p7
itabl1 = p8

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0
kfmenv linseg .2, .2*p3, 2, .6*p3, .5, .2*p3, .2

; Frequency Sweep
kfco linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez/sqrt(kfco)-1
ka2 = 1000/kfco

; Initialize Yn-1 & Yn-2 to zero
aynm1 init 0
aynm2 init 0

; Oscillator
  axn foscil iamp, ifqc, p9, p10, kfmenv, itabl1

; Replace the differential eq. with a difference eq.
  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)
  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr delay aoutl, .01
  outs aoutl, aoutr

endin

;******************************************************************
instr 2   ; 4 Pole Filter

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez1  = p7
irez2  = p9
itabl1 = p10

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0

; Frequency Sweep
kfco1 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6
kfco2 linseg .1*p8, .5*p3, p8, .5*p3, .1*p8

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez1/sqrt(kfco1)-1
ka2 = 1000/kfco1
kb1 = 100/irez2/sqrt(kfco2)-1
kb2 = 1000/kfco2

; Initialize Yn-1 to Yn-4 to zero
ayn1m1 init 0
ayn1m2 init 0
ayn2m1 init 0
ayn2m2 init 0

; Oscillator
  axn oscil iamp, ifqc, itabl1

; Replace the differential eq. with a difference eq.
  ayn1 = ((ka1+2*ka2)*ayn1m1-ka2*ayn1m2+axn)/(1+ka1+ka2)

  ayn1m2 = ayn1m1
  ayn1m1 = ayn1

  ayn = ((kb1+2*kb2)*ayn2m1-kb2*ayn2m2+ayn1)/(1+kb1+kb2)

  ayn2m2 = ayn2m1
  ayn2m1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr delay aoutl, .01
  outs aoutl, aoutr

endin

;******************************************************************
instr 3   ; 6 Pole Formant Filter

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez1  = p7
irez2  = p9
irez3  = p11
itabl1 = p12

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0

; Frequency Sweep
kfco1 linseg .1*p6,  .5*p3, p6,  .5*p3, .1*p6
kfco2 linseg .1*p8,  .5*p3, p8,  .5*p3, .1*p8
kfco3 linseg .1*p10, .5*p3, p10, .5*p3, .1*p10

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez1/sqrt(kfco1)-1
ka2 = 1000/kfco1
kb1 = 100/irez2/sqrt(kfco2)-1
kb2 = 1000/kfco2
kc1 = 100/irez3/sqrt(kfco3)-1
kc2 = 1000/kfco3

; Initialize Yn-1 to Yn-2 to zero
ayn1m1 init 0
ayn1m2 init 0
ayn2m1 init 0
ayn2m2 init 0
ayn3m1 init 0
ayn3m2 init 0

; Oscillator
  axn oscil iamp, ifqc, itabl1

; Replace the differential eq. with a difference eq.
  ayn1 = ((ka1+2*ka2)*ayn1m1-ka2*ayn1m2+axn)/(1+ka1+ka2)

  ayn1m2 = ayn1m1
  ayn1m1 = ayn1

  ayn2 = ((kb1+2*kb2)*ayn2m1-kb2*ayn2m2+ayn1)/(1+kb1+kb2)

  ayn2m2 = ayn2m1
  ayn2m1 = ayn2

  ayn = ((kc1+2*kc2)*ayn3m1-kc2*ayn3m2+ayn2)/(1+kc1+kc2)

  ayn3m2 = ayn3m1
  ayn3m1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr delay aoutl, .01
  outs aoutl, aoutr

endin

;******************************************************************
instr 4   ; Mandelbrot's Snowflake 4 Pole Filter

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez1  = p7
irez2  = p7
iamp2  = p8
iamp3  = p9

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0

; Frequency Sweep
kfco1 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6
kfco2 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez1/sqrt(kfco1)-1
ka2 = 1000/kfco1
kb1 = 100/irez2/sqrt(kfco2)-1
kb2 = 1000/kfco2

; Initialize Yn-1 to Yn-4 to zero
ayn1m1 init 0
ayn1m2 init 0
ayn2m1 init 0
ayn2m2 init 0

; Oscillator
  ax1 oscil iamp,  ifqc, 6
  ax2 oscil iamp2, ifqc*4, 6
  ax3 oscil iamp3, ifqc/4, 6
  axn = ax1 + ax2 + ax3

; Replace the differential eq. with a difference eq.
  ayn1 = ((ka1+2*ka2)*ayn1m1-ka2*ayn1m2+axn)/(1+ka1+ka2)

  ayn1m2 = ayn1m1
  ayn1m1 = ayn1

  ayn = ((kb1+2*kb2)*ayn2m1-kb2*ayn2m2+ayn1)/(1+kb1+kb2)

  ayn2m2 = ayn2m1
  ayn2m1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr = axn * kaenv
  outs aoutl, aoutr

endin

;******************************************************************
instr 5   ; Snowflake 4 Pole Filter

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez1  = p7
irez2  = p7
iamp2  = p8
iamp3  = p9
iamp4  = p10
iamp5  = p11

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0

; Frequency Sweep
kfco1 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6
kfco2 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez1/sqrt(kfco1)-1
ka2 = 1000/kfco1
kb1 = 100/irez2/sqrt(kfco2)-1
kb2 = 1000/kfco2

; Initialize Yn-1 to Yn-4 to zero
ayn1m1 init 0
ayn1m2 init 0
ayn2m1 init 0
ayn2m2 init 0

; Oscillator
  ax1 oscil iamp,  ifqc, 4
  ax2 oscil iamp2, ifqc*2, 4
  ax3 oscil iamp3, ifqc*4, 4
  ax4 oscil iamp3, ifqc/2, 4
  ax5 oscil iamp3, ifqc/4, 4
  axn = ax1 + ax2 + ax3 + ax4 + ax5

; Replace the differential eq. with a difference eq.
  ayn1 = ((ka1+2*ka2)*ayn1m1-ka2*ayn1m2+axn)/(1+ka1+ka2)

  ayn1m2 = ayn1m1
  ayn1m1 = ayn1

  ayn = ((kb1+2*kb2)*ayn2m1-kb2*ayn2m2+ayn1)/(1+kb1+kb2)

  ayn2m2 = ayn2m1
  ayn2m1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr = axn * kaenv
  outs aoutl, aoutr

endin

;******************************************************************
instr 6   ; Sample Hold Generator

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez1  = p7
irez2  = p7
iamp2  = p8
iamp3  = p9
iamp4  = p10
iamp5  = p11

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0

; Frequency Sweep
kfco1 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6
kfco2 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez1/sqrt(kfco1)-1
ka2 = 1000/kfco1
kb1 = 100/irez2/sqrt(kfco2)-1
kb2 = 1000/kfco2

; Initialize Yn-1 to Yn-4 to zero
ayn1m1 init 0
ayn1m2 init 0
ayn2m1 init 0
ayn2m2 init 0

; Oscillator
  axn randi iamp, ifqc

; Replace the differential eq. with a difference eq.
  ayn1 = ((ka1+2*ka2)*ayn1m1-ka2*ayn1m2+axn)/(1+ka1+ka2)

  ayn1m2 = ayn1m1
  ayn1m1 = ayn1

  ayn = ((kb1+2*kb2)*ayn2m1-kb2*ayn2m2+ayn1)/(1+kb1+kb2)

  ayn2m2 = ayn2m1
  ayn2m1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr = ayn * kaenv
  outs aoutl, aoutr

endin

;******************************************************************
instr 7   ; Saw/Ramp /\ Transformation

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez1  = p7
itabl1 = p8

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.03, 1, .01, 0, .01, 0

; Frequency Sweep
kfco1 linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez1/sqrt(kfco1)-1
ka2 = 1000/kfco1

; Initialize Yn-1 to Yn-4 to zero
aynm1 init 0
aynm2 init 0
adx   init 20000/sr*ifqc

; Oscillator
start:
  timout 0, 1/ifqc, continue
  axn = -10000
  kdx = -kdx
  kslope = (kdx<0 ? 10 : 1/10)
  
  reinit start

; Replace the differential eq. with a difference eq.
continue: axn = axn + kdx

  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)
  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  aoutl = ayn * kaenv
  aoutr = ayn * kaenv
  outs aoutl, aoutr

endin
