;---------------------------------------------------------------
; A set of analog modeling instruments using resonant low-pass
; filter by Hans Mikelson November 1998
;---------------------------------------------------------------
sr = 44100
kr = 4410
ksmps = 10
nchnls = 2
zakinit 50, 50

;---------------------------------------------------------------
; LFO
;---------------------------------------------------------------
           instr   5

idur       =       p3
iamp       =       p4
ifqc       =       p5
iwave      =       p6
ioffset    =       p7
ioutch     =       p8

ksig       oscil   iamp, ifqc, iwave
kout       =       ksig*.5+ioffset
           zkw     kout, ioutch

           endin

;---------------------------------------------------------------
; Moog VCO with Mikleson filter
;---------------------------------------------------------------
           instr   10

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
ifqc       =       cpspch(p5)      ; Frequency
irtfqc     =       sqrt(ifqc)      ; Square root of frequency
ifco       =       p6*ifqc/500     ; Filter cut off (higher frequencies get a higher cut off)
krez       init    p7              ; Resonance does not change in this instrument
iwave      =       p8              ; Selected wave form 1=Saw, 2=Square/PWM, 3=Tri/Saw-Ramp-Mod
ipwch      =       p9              ; PWM SRM modulation channel
ileak      =       (iwave==2 ? .999 : (iwave==3 ? .995 : .999)) ; Leaky integration value

kpw        zkr     ipwch           ; Read the Zak modulation channel
kpwn       =       kpw-.5          ; Get a pulse width that goes from -.5 to .5

kfco       expseg  100+.01*ifco, .2*idur, ifco+100, .5*idur, ifco*.1+100, .3*idur, .001*ifco+100 ; Filter envelope

apulse1    buzz    1, ifqc, sr/2/ifqc, 1 ; Band-limited impulse train
apulse2    vdelay  apulse1, 1000/ifqc*kpw, 1000/ifqc ; Delay the impulse train by lambda/pw.

asaw       biquad  apulse1, 1, 0, 0, 1, -ileak, 0    ; Biquad is used to do a leaky integrator for saw wave
apdiff     =       apulse1-apulse2                   ; Pulse difference is needed for square wave
asqr1      biquad  apdiff,  1, 0, 0, 1, -ileak, 0    ; Biquad used for leaky integrator of pulse difference
asqr       =       asqr1+kpwn                        ; Adjustment to remove DC component
atri1      biquad  asqr1,   1, 0, 0, 1, -ileak, 0    ; Triangle wave, many adjustments are necessary to conteract amplitude
atri       =      (atri1*irtfqc/(kpw*(1-kpw))/3000+5*kpwn/irtfqc)*(1-kpwn*kpwn*20/irtfqc)/sr*44100 ; and DC variations

asig       =       (iwave==2 ? asqr : (iwave==3 ? atri : asaw)) ; Select your waveform here

;aout       rezzy   asig, kfco, krez                             ; Apply the filter

           outs    asig*iamp, asig*iamp                         ; Stereo ouput and amplification
           endin


;---------------------------------------------------------------
; Moog VCO with Mikleson filter
;---------------------------------------------------------------
           instr   11

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
ifqc       =       cpspch(p5)      ; Frequency
irtfqc     =       sqrt(ifqc)      ; Square root of frequency
ifco       =       p6*ifqc/500     ; Filter cut off (higher frequencies get a higher cut off)
krez       init    p7              ; Resonance does not change in this instrument
iwave      =       p8              ; Selected wave form 1=Saw, 2=Square/PWM, 3=Tri/Saw-Ramp-Mod
ipwch      =       p9              ; PWM Channel
ipbend     =       p10             ; Pitch Bend Table

ileak      =       (iwave==2 ? .999 : (iwave==3 ? .995 : .999)) ; Leaky integration value

kpw        zkr     ipwch           ; Read the Zak modulation channel
kpwn       =       kpw-.5          ; Get a pulse width that goes from -.5 to .5
kfco       expseg  100+.1*ifco, .2*idur, ifco+100, .5*idur, ifco*.1+100, .3*idur, .1*ifco+100 ; Filter envelope
kpbend     oscil   1, 1/idur, ipbend  ; Pitch bend
kamp       linseg  0, .01*idur, iamp, .2*idur, .8*iamp, .59*idur, .5*iamp, .2*idur, 0            ; Amp envelope
kfqc       =       ifqc*kpbend

apulse1    buzz    1, kfqc, sr/2/kfqc, 1 ; Band-limited impulse train
apulse2    vdelay  apulse1, 1000/kfqc*kpw, 1000 ; Delay the impulse train by lambda/pw.

asaw       biquad  apulse1, 1, 0, 0, 1, -ileak, 0    ; Biquad is used to do a leaky integrator for saw wave
apdiff     =       apulse1-apulse2                   ; Pulse difference is needed for square wave
asqr1      biquad  apdiff,  1, 0, 0, 1, -ileak, 0    ; Biquad used for leaky integrator of pulse difference
asqr       =       asqr1+kpwn                        ; Adjustment to remove DC component
atri1      biquad  asqr1,   1, 0, 0, 1, -ileak, 0    ; Triangle wave, many adjustments are necessary to conteract amplitude
atri       =      (atri1*irtfqc/(kpw*(1-kpw))/3000+5*kpwn/irtfqc)*(1-kpwn*kpwn*20/irtfqc)/sr*44100 ; and DC variations

asig       =       (iwave==2 ? asqr : (iwave==3 ? atri : asaw)) ; Select your waveform here

aout       rezzy   asig, kfco*kpbend, krez                      ; Apply the filter

           outs    asig*iamp, asig*iamp                         ; Stereo ouput and amplification
           endin

;---------------------------------------------------------------
; Moog VCO with Mikleson filter
;---------------------------------------------------------------
           instr   12

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
ifqc       =       cpspch(p5)      ; Frequency
irtfqc     =       sqrt(ifqc)      ; Square root of frequency
ifco       =       p6*ifqc/500     ; Filter cut off (higher frequencies get a higher cut off)
krez       init    p7              ; Resonance does not change in this instrument
iwave      =       p8              ; Selected wave form 1=Saw, 2=Square/PWM, 3=Tri/Saw-Ramp-Mod
ipwch      =       p9              ; PWM Channel
ipbend     =       p10             ; Pitch Bend Table
imode      =       p11             ; Filter Mode
isine      =       1
imaxd      =       1/ifqc*2        ; Allows pitch bend down of two octaves
ioutch1    =       p12
ioutch2    =       p13

ileak      =       (iwave==2 ? .999 : (iwave==3 ? .995 : .999)) ; Leaky integration value

kpw        zkr     ipwch           ; Read the Zak modulation channel
kpwn       =       kpw-.5          ; Get a pulse width that goes from -.5 to .5
kfco       expseg  100+.1*ifco, .2*idur, ifco+100, .5*idur, ifco*.1+100, .3*idur, .1*ifco+100 ; Filter envelope
kpbend     oscil   1, 1/idur, ipbend  ; Pitch bend
kamp       linseg  0, .01*idur, iamp, .2*idur, .8*iamp, .69*idur, .7*iamp, .1*idur, 0            ; Amp envelope
kfqc       =       ifqc*kpbend

asig1      vco     1, kfqc*1.001, iwave, kpw, 1, imaxd
asig2      vco     1, kfqc*0.999, iwave, kpw, 1, imaxd

kfqc2      linseg  100, idur, 1000

afco1      oscil   .5, kfqc2, 1
arez       =       .99 ;(1+afco1)/2
afco       =       (1+afco1)*1000

aout1      moogvcf asig1, kfco, arez ;, imode                ; Apply the filter
aout2      moogvcf asig2, kfco, arez ;, imode                ; Apply the filter

           outs    aout1*kamp, aout2*kamp                         ; Stereo ouput and amplification
           zaw     aout1*kamp, ioutch1
           zaw     aout2*kamp, ioutch2
           endin

;---------------------------------------------------------------
; Delay
;---------------------------------------------------------------
           instr   50

idur       =       p3              ; Duration
iamp       =       p4              ; Amplitude
itim1      =       p5              ; Time 1
itim2      =       p6              ; Time 2
ifdbk1     =       p7              ; Feedback 1
ifdbk2     =       p8              ; Feedback 2
ixfdbk1    =       p9              ; Cross FeedBack 1
ixfdbk2    =       p10             ; Cross FeedBack 2
iinch1     =       p11             ; In Channel 1
iinch2     =       p12             ; In Channel 2

adel1      init    0
adel2      init    0

asig1      zar     iinch1
asig2      zar     iinch2

adelt      delay   asig1*iamp+ifdbk1*adel1+ixfdbk2*adel2, itim1
adel2      delay   asig2*iamp+ifdbk2*adel2+ixfdbk1*adel1, itim2

adel1      =       adelt

           outs    adel1, adel2

           endin

